﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GME.Common.UI.Helpers
{
    using System.Windows;
    using System.Windows.Documents;
    using System.Windows.Media;

    public class VisualTreeHelperHelper
    {
        public static T GetParent<T>(DependencyObject element) where T : DependencyObject
        {
            var parent = element;
            while (parent != null && !(parent is T))
            {
                parent = VisualTreeHelper.GetParent(parent);
            }

            return parent as T;
        }

        public static DependencyObject GetTopParent(DependencyObject element)
        {
            var parent = element;
            while (parent != null && !(parent is AdornerDecorator))
            {
                parent = VisualTreeHelper.GetParent(parent);
            }

            var ad = parent as AdornerDecorator;
            if (ad != null)
                return ad.Child;

            return parent;
        }

        public static T GetParentFromWindowedChild<T>(DependencyObject element) where T : DependencyObject
        {
            var parent = element;
            while (parent != null && !(parent is T))
            {
                if (parent is Window)
                {
                    var window = parent as Window;
                    parent = window.Owner;
                }
                else
                {
                    parent = VisualTreeHelper.GetParent(parent);
                }
            }

            return parent as T;
        }

        public static T GetChild<T>(DependencyObject element) where T : DependencyObject
        {
            if (element is T)
                return element as T;

            var childCount = VisualTreeHelper.GetChildrenCount(element);
            for(int i=0; i<childCount; ++i)
            {
                var child = VisualTreeHelper.GetChild(element, i);
                var childRes = GetChild<T>(child);

                if (childRes != null)
                    return childRes;
            }

            return null;
        }

        public static void GetChildren<T>(DependencyObject element, List<T> returnList) where T : DependencyObject
        {
            if (element is T)
            {
                returnList.Add(element as T);
                return;
            }

            var childCount = VisualTreeHelper.GetChildrenCount(element);
            for (int i = 0; i < childCount; ++i)
            {
                var child = VisualTreeHelper.GetChild(element, i);
                GetChildren<T>(child, returnList);
            }
        }

        public static void GetChildrenFromOwnedWindows<T>(DependencyObject element, List<T> returnList) where T : DependencyObject
        {
            var window = GetParent<Window>(element);

            foreach(Window childWindow in window.OwnedWindows)
            {
                GetChildren<T>(childWindow, returnList);
            }
        }

        public static T GetNamedChildFromOwnedWindows<T>(DependencyObject element, string name) where T : DependencyObject
        {
            var window = GetParent<Window>(element);

            foreach (Window childWindow in window.OwnedWindows)
            {
                var child = GetNamedChild<T>(childWindow, name);
                if (child != null)
                    return child;
            }

            return null;
        }

        public static T GetNamedChild<T>(DependencyObject element, string name) where T : DependencyObject
        {
            if (element is T && element is FrameworkElement && name.Equals(((FrameworkElement)element).Name))
                return element as T;

            var childCount = VisualTreeHelper.GetChildrenCount(element);
            for (int i = 0; i < childCount; ++i)
            {
                var child = VisualTreeHelper.GetChild(element, i);
                var childRes = GetNamedChild<T>(child, name);

                if (childRes != null)
                    return childRes;
            }

            return null;
        }

        public static List<T> GetNamedChildrenStartingWith<T>(DependencyObject element, string startsWith)
    where T : DependencyObject
        {
            var result = new List<T>();
            GetNamedChildrenStartingWith(element, startsWith, result);
            return result;
        }

        private static void GetNamedChildrenStartingWith<T>(DependencyObject element, string startsWith, List<T> result)
            where T : DependencyObject
        {
            if (element is T && element is FrameworkElement && ((FrameworkElement)element).Name.StartsWith(startsWith))
                result.Add(element as T);

            var childCount = VisualTreeHelper.GetChildrenCount(element);
            for (int i = 0; i < childCount; ++i)
            {
                var child = VisualTreeHelper.GetChild(element, i);
                GetNamedChildrenStartingWith<T>(child, startsWith, result);
            }
        }


    }
}
